package org.example.wedding_app_backend.api.controller;

import org.assertj.db.type.Table;
import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingConfirmationGroup.entity.WeddingConfirmationGroupEntity;
import org.example.wedding_app_backend.weddingConfirmationGroup.repository.WeddingConfirmationGroupRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;

import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.repository.WeddingPresentRepository;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithAnonymousUser;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import javax.sql.DataSource;

import java.time.LocalDateTime;
import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc
public class WeddingConfirmationGroupControllerTests {

    @Autowired
    private MockMvc mockMvc;


    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingConfirmationGroupRepository weddingConfirmationGroupRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;

    @MockBean
    private DataInitializer dataInitializer;//Disable data initialization

    @Autowired
    DataSource dataSource;

    @AfterEach
    void clean() {
        weddingConfirmationGroupRepository.deleteAll();
        weddingGuestRepository.deleteAll();
        weddingRepository.deleteAll();
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void getWeddingConfirmationGroupData_ShouldReturnGroups_WhenUserHasWedding() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .wedding(wedding_1)
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .presenceConfirmed(null)
                .guestName("AREK")
                .guestAge("Dorosły")
                .weddingConfirmationGroup(group_1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        @Language("JSON")
        String expected = """
                [
                  {
                    "id": "%s",
                    "name": "Kowalscy",
                    "guestList": [
                      {
                        "id": "%s",
                        "guestName": "AREK",
                        "guestAge": "Dorosły",
                        "presenceConfirmed": null
                      }
                    ]
                  }
                ]
                """.formatted(
                UUID.nameUUIDFromBytes("group_1".getBytes()),
                UUID.nameUUIDFromBytes("wedding_guest_1".getBytes())
        );

        mockMvc.perform(get("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void getWeddingConfirmationGroupData_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    @WithAnonymousUser
    void getWeddingConfirmationGroupData_ShouldReturnUnauthorized_WhenUserIsAnonymous() throws Exception {
        mockMvc.perform(get("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void createWeddingGuest_ShouldCreatePresent_WhenValidRequest() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        String newGroupJson = """
                {
                   "id": "%s",
                   "name": "Kowalscy"
                    }
                """.formatted(
                UUID.nameUUIDFromBytes("group_1".getBytes())
        );

        mockMvc.perform(post("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newGroupJson))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_confirmation_groups"))
                .hasNumberOfRows(1)
                .row()
                .column("name").value().isEqualTo("Kowalscy");
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void createWeddingGuest_ShouldReturnNotFound_WhenWeddingDoesNotExist() throws Exception {
        String newGroupJson = """
                {
                   "id": "%s",
                   "name": "Kowalscy"
                    }
                """.formatted(
                UUID.nameUUIDFromBytes("group_1".getBytes())
        );

        mockMvc.perform(post("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newGroupJson))
                .andExpect(status().isNotFound());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void deleteWeddingGroup_ShouldDeletePresent_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .wedding(wedding_1)
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        mockMvc.perform(delete("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"id\": \"" + group_1.getId() + "\"}"))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_confirmation_groups"))
                .hasNumberOfRows(0);

    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void deleteWeddingGroup_ShouldReturnNotFound_WhenPresentDoesNotExist() throws Exception {

        mockMvc.perform(delete("/api/users/groups/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"id\": \"123e4567-e89b-12d3-a456-426614174000\"}"))
                .andExpect(status().isNotFound());
    }

}
