package org.example.wedding_app_backend.api.controller;

import org.assertj.db.type.Table;
import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingConfirmationGroup.entity.WeddingConfirmationGroupEntity;
import org.example.wedding_app_backend.weddingConfirmationGroup.repository.WeddingConfirmationGroupRepository;
import org.example.wedding_app_backend.weddingConfirmationGroup.service.WeddingConfirmationGroupService;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;

import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.repository.WeddingPresentRepository;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import org.springframework.http.MediaType;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.test.context.support.WithAnonymousUser;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import javax.sql.DataSource;
import java.time.LocalDateTime;

import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc
public class WeddingGuestControllerTests {

    @Autowired
    private MockMvc mockMvc;


    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;

    @Autowired
    private WeddingConfirmationGroupRepository weddingConfirmationGroupRepository;

    @MockBean
    private DataInitializer dataInitializer;//Disable data initialization

    @Autowired
    DataSource dataSource;

    @AfterEach
    void clean() {
        weddingConfirmationGroupRepository.deleteAll();
        weddingGuestRepository.deleteAll();
        weddingRepository.deleteAll();
    }

    @Test
    void getWeddingGuestQuizState_ShouldReturnState_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .quizFinished(false)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        @Language("JSON")
        String expected = """
                  {
                    "quizFinished": false
                  }
                """;

        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/quiz/state",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes())))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWeddingGuestQuizState_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/quiz/state",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void getWeddingGuestQuizScore_ShouldReturnState_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .quizPoints(1)
                .questionIndex(1)
                .guestName("AREK")
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        @Language("JSON")
        String expected = """
                  {
                    "questionIndex": 1,
                    "quizPoints": 1,
                    "guestName": "AREK"
                  }
                """;

        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/quiz/score",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes())))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWeddingGuestQuizScore_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/quiz/score",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void getWeddingGuestsFinalQuizScore_ShouldReturnState_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .quizPoints(1)
                .questionIndex(1)
                .quizFinished(true)
                .guestName("AREK")
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        WeddingGuestEntity guest_2 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_2".getBytes()))
                .quizPoints(2)
                .questionIndex(1)
                .quizFinished(true)
                .guestName("DAREK")
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_2);

        @Language("JSON")
        String expected = """
            [
              {
                "id": "%s",
                "quizPoints": 1,
                "guestName": "AREK"
              },
              {
                "id": "%s",
                "quizPoints": 2,
                "guestName": "DAREK"
              }
            ]
            """.formatted(
                UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                UUID.nameUUIDFromBytes("wedding_guest_2".getBytes())
        );

        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/games/hall",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes())))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWeddingGuestsFinalQuizScore_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/games/hall",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void getWhoKnowsBetterQuizState_ShouldReturnState_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .whoKnowsQuizFinished(false)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        @Language("JSON")
        String expected = """
                  {
                    "whoKnowsQuizFinished": false
                  }
                """;

        mockMvc.perform(get("/api/wedding/{weddingId}/{guestId}/games/who-knows-better/state",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes())))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWhoKnowsBetterQuizState_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/wedding/{weddingId}/{guestId}/games/who-knows-better/state",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void getWeddingGuestWithGroupInformation_ShouldReturn_WhenUserHasWedding() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .presenceConfirmed(null)
                .guestName("AREK")
                .guestAge("Dorosły")
                .weddingConfirmationGroup(group_1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        WeddingGuestEntity guest_2 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_2".getBytes()))
                .presenceConfirmed(null)
                .guestName("DAREK")
                .guestAge("Dorosły")
                .weddingConfirmationGroup(group_1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_2);

        @Language("JSON")
        String expected = """
                [
                    {
                        "id": "%s",
                        "presenceConfirmed": null,
                        "guestName": "AREK",
                        "guestAge": "Dorosły",
                        "weddingConfirmationGroup": {
                            "id": "%s",
                            "name": "Kowalscy"
                        }
                    },
                    {
                        "id": "%s",
                        "presenceConfirmed": null,
                        "guestName": "DAREK",
                        "guestAge": "Dorosły",
                        "weddingConfirmationGroup": {
                            "id": "%s",
                            "name": "Kowalscy"
                        }
                    }
                ]
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                UUID.nameUUIDFromBytes("group_1".getBytes()),
                UUID.nameUUIDFromBytes("wedding_guest_2".getBytes()),
                UUID.nameUUIDFromBytes("group_1".getBytes())
        );

        mockMvc.perform(get("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void getWeddingGuestWithGroupInformation_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    @WithAnonymousUser
    void getWeddingGuestWithGroupInformation_ShouldReturnUnauthorized_WhenUserIsAnonymous() throws Exception {
        mockMvc.perform(get("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void updateWeddingGuestParametersModifiableByUser_ShouldUpdate_WhenPresentExist() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .presenceConfirmed(null)
                .guestName("AREK")
                .guestAge("Dorosły")
                .weddingConfirmationGroup(group_1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);


        String updatedGuestJson = """
                {
                        "id": "%s",
                        "presenceConfirmed": null,
                        "guestName": "MAREK",
                        "guestAge": "Dorosły",
                        "weddingConfirmationGroup": {
                            "id": "%s",
                            "name": "Kowalscy"
                        }
                    }
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                UUID.nameUUIDFromBytes("group_1".getBytes())
                );

        mockMvc.perform(patch("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(updatedGuestJson))
                .andExpect(status().isOk());

        assertThat(new Table(dataSource, "wedding_guests"))
                .hasNumberOfRows(1)
                .row()
                .column("guest_Name").value().isEqualTo("MAREK");
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void updateWeddingGuestParametersModifiableByUser_ShouldReturnNotFound_WhenWeddingDoesNotExist() throws Exception {
        String newGuestJson = """
                {
                        "id": "%s",
                        "presenceConfirmed": null,
                        "guestName": "MAREK",
                        "guestAge": "Dorosły",
                        "weddingConfirmationGroup": {
                            "id": "%s",
                            "name": "Kowalscy"
                        }
                    }
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                UUID.nameUUIDFromBytes("group_1".getBytes())
        );

        mockMvc.perform(patch("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newGuestJson))
                .andExpect(status().isNotFound());
    }

    @Test
    @WithAnonymousUser
    void updateWeddingGuestParametersModifiableByUser_ShouldReturnUnauthorized_WhenUserIsAnonymous() throws Exception {
        mockMvc.perform(patch("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void createWeddingGuest_ShouldCreatePresent_WhenValidRequest() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        String newGuestJson = """
                {
                        "id": null,
                        "presenceConfirmed": null,
                        "guestName": "MAREK",
                        "guestAge": "Dorosły",
                        "weddingConfirmationGroup": {
                            "id": "%s",
                            "name": "Kowalscy"
                        }
                    }
                """.formatted(
                UUID.nameUUIDFromBytes("group_1".getBytes())
        );

        mockMvc.perform(post("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newGuestJson))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_guests"))
                .hasNumberOfRows(1)
                .row()
                .column("guest_Name").value().isEqualTo("MAREK");
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void createWeddingGuest_ShouldReturnNotFound_WhenWeddingDoesNotExist() throws Exception {
        String newGuestJson = """
                {
                        "id": "%s",
                        "presenceConfirmed": null,
                        "guestName": "MAREK",
                        "guestAge": "Dorosły",
                        "weddingConfirmationGroup": {
                            "id": "%s",
                            "name": "Kowalscy"
                        }
                    }
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                UUID.nameUUIDFromBytes("group_1".getBytes())
        );

        mockMvc.perform(post("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newGuestJson))
                .andExpect(status().isNotFound());
    }

    @Test
    @WithAnonymousUser
    void createWeddingGuest_ShouldReturnUnauthorized_WhenUserIsAnonymous() throws Exception {
        mockMvc.perform(post("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void deleteWeddingGuest_ShouldDeletePresent_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .presenceConfirmed(null)
                .guestName("AREK")
                .guestAge("Dorosły")
                .weddingConfirmationGroup(group_1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        mockMvc.perform(delete("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"id\": \"" + guest_1.getId() + "\"}"))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_guests"))
                .hasNumberOfRows(0);

    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void deleteWeddingGuest_ShouldReturnNotFound_WhenPresentDoesNotExist() throws Exception {

        mockMvc.perform(delete("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"id\": \"123e4567-e89b-12d3-a456-426614174000\"}"))
                .andExpect(status().isNotFound());
    }

    @Test
    @WithAnonymousUser
    void deleteWeddingGuest_ShouldReturnUnauthorized_WhenUserIsAnonymous() throws Exception {
        mockMvc.perform(delete("/api/users/guests")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    void updateWeddingGuestPresenceConfirmation_ShouldUpdate_WhenPresentExist() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingConfirmationGroupEntity group_1 = WeddingConfirmationGroupEntity.builder()
                .id(UUID.nameUUIDFromBytes("group_1".getBytes()))
                .name("Kowalscy")
                .build();
        weddingConfirmationGroupRepository.save(group_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .presenceConfirmed(null)
                .guestName("AREK")
                .guestAge("Dorosły")
                .weddingConfirmationGroup(group_1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        mockMvc.perform(patch("/api/wedding/{weddingId}/guests/{guestId}",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("presenceConfirmed", "true"))
                .andExpect(status().isOk());

        assertThat(new Table(dataSource, "wedding_guests"))
                .hasNumberOfRows(1)
                .row()
                .column("presence_Confirmed").value().isEqualTo(true);
    }

    @Test
    void updateWeddingGuestPresenceConfirmation_ShouldReturnNotFound_WhenWeddingDoesNotExist() throws Exception {

        mockMvc.perform(patch("/api/wedding/{weddingId}/guests/{guestId}",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON)
                        .param("presenceConfirmed", "true"))
                .andExpect(status().isNotFound());
    }

}

