package org.example.wedding_app_backend.api.controller;

import org.assertj.db.type.Table;
import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;

import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.repository.WeddingPresentRepository;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import org.springframework.http.MediaType;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.test.context.support.WithAnonymousUser;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import javax.sql.DataSource;
import java.time.LocalDateTime;

import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc
public class WeddingPresentControllerTests {

    @Autowired
    private MockMvc mockMvc;


    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingPresentRepository weddingPresentRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;

    @MockBean
    private DataInitializer dataInitializer;//Disable data initialization

    @Autowired
    DataSource dataSource;

    @AfterEach
    void clean() {
        weddingPresentRepository.deleteAll();
        weddingGuestRepository.deleteAll();
        weddingRepository.deleteAll();
    }


    @Test
    void getWeddingPresents_ShouldReturnPresents_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .reservedBy(UUID.nameUUIDFromBytes("guest_1".getBytes()))
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_2".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_3".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        @Language("JSON")
        String expected = """
                [
                  {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present duh",
                    "reservedBy": "%s"
                  },
                 {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present duh",
                    "reservedBy": null
                  },
                  {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present duh",
                    "reservedBy": null
                  }
                ]
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_present_1".getBytes()),
                UUID.nameUUIDFromBytes("guest_1".getBytes()),
                UUID.nameUUIDFromBytes("wedding_present_2".getBytes()),
                UUID.nameUUIDFromBytes("wedding_present_3".getBytes())
        );

        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/presents",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("guest_1".getBytes())))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWeddingPresents_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/presents",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void updateWeddingPresentReservedBy_ShouldUpdatePresentReservedBy_WhenExists() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        String updatedPresentJson = """
                {
                    "id": "%s",
                    "name": "Present2",
                    "linkToPresent": "http:/test2",
                    "description": "Updated Present",
                    "reservedBy": "%s"
                }
                """.formatted(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()),
                UUID.nameUUIDFromBytes("guest_1".getBytes())
                );

        mockMvc.perform(patch("/api/weddings/{weddingId}/{guestId}/presents",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("guest_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(updatedPresentJson))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_presents"))
                .hasNumberOfRows(1)
                .row()
                .column("reserved_By").value().isEqualTo(UUID.nameUUIDFromBytes("guest_1".getBytes()));
    }

    //user side

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void getWeddingPresentsForUser_ShouldReturnPresents_WhenUserHasWedding() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_2".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        @Language("JSON")
        String expected = """
                [
                  {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present duh",
                    "reservedBy": null
                  },
                 {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present duh",
                    "reservedBy": null
                  }
                ]
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_present_1".getBytes()),
                UUID.nameUUIDFromBytes("wedding_present_2".getBytes())
        );

        mockMvc.perform(get("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void getWeddingPresentsForUser_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    @WithAnonymousUser
    void getWeddingPresentsForUser_ShouldReturnUnauthorized_WhenUserIsAnonymous() throws Exception {
        mockMvc.perform(delete("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void updateWeddingPresentParametersModifiableByUser_ShouldUpdate_WhenPresentExist() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        String updatedPresentJson = """
                {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present UPDATEDDD",
                    "reservedBy": null
                  }
                """.formatted(
                UUID.nameUUIDFromBytes("wedding_present_1".getBytes()));

        mockMvc.perform(patch("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(updatedPresentJson))
                .andExpect(status().isOk());

        assertThat(new Table(dataSource, "wedding_presents"))
                .hasNumberOfRows(1)
                .row()
                .column("description").value().isEqualTo("Present UPDATEDDD");
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void updateWeddingPresentParametersModifiableByUser_ShouldReturnNotFound_WhenWeddingDoesNotExist() throws Exception {
        String newPresentJson = """
                {
                    "id": "%s",
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present UPDATEDDD",
                    "reservedBy": null
                  }
                """.formatted(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()));

        mockMvc.perform(patch("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newPresentJson))
                .andExpect(status().isNotFound());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void createWeddingPresent_ShouldCreatePresent_WhenValidRequest() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        String newPresentJson = """
                {
                    "id": null,
                    "name": "Present",
                    "linkToPresent": "http:/test",
                    "description": "Present",
                    "reservedBy": null
                  }
                """;

        mockMvc.perform(post("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newPresentJson))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_presents"))
                .hasNumberOfRows(1)
                .row()
                .column("description").value().isEqualTo("Present");
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void createWeddingPresent_ShouldReturnNotFound_WhenWeddingDoesNotExist() throws Exception {
        String newPresentJson = """
                {
                    "description": "New Present",
                    "reservedBy": null
                }
                """;

        mockMvc.perform(post("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(newPresentJson))
                .andExpect(status().isNotFound());
    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void deleteWeddingPresent_ShouldDeletePresent_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .userId(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        weddingPresentRepository.save(WeddingPresentEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_present_1".getBytes()))
                .name("Present")
                .linkToPresent("http:/test")
                .description("Present duh")
                .wedding(wedding_1)
                .build());

        WeddingPresentEntity present = weddingPresentRepository.findById(UUID.nameUUIDFromBytes("wedding_present_1".getBytes())).get();

        mockMvc.perform(delete("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"id\": \"" + present.getId() + "\"}"))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_presents"))
                .hasNumberOfRows(0);

    }

    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void deleteWeddingPresent_ShouldReturnNotFound_WhenPresentDoesNotExist() throws Exception {

        mockMvc.perform(delete("/api/users/presents")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"id\": \"123e4567-e89b-12d3-a456-426614174000\"}"))
                .andExpect(status().isNotFound());
    }

}

